#!/usr/bin/php
<?php
include_once('lib/HttpClient.class.php');

//$DEFAULT_SRV_URL = 'http://127.0.0.1:2082';
$DEFAULT_TIMEOUT = 50;
$DEFAULT_CONF_FILE = '/etc/esia/rcm.ini';
$DEFAULT_TYPE = 'ps1';
$DEFAULT_METHOD = 'POST';

$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

$conf = parse_ini_file($DEFAULT_CONF_FILE,true);
$o_key = $conf['LISTEN']['key'];
$o_server_url = 'http://'.$conf['LISTEN']['address'].':'.$conf['LISTEN']['port'];

//Récupération et vérifications des paramètres.
$type_list = array('load','mem','storage','uptime','time','interface','iodisk','smartctl','service','scheduled_task');

$params = new Parameters();
$params->addOpt('','h','help',NULL,'print this help');
$params->addOpt('','D','debug',NULL,'Use to enable debug mode');
$params->addOpt(':','u','uri','<url>','Powershell daemon url (http://127.0.0.1:2082 by default)');
$params->addOpt(':','H','host','<host>','Host IP ou FQDN');
$params->addOpt(':','U','user','<login>','Login or username');
$params->addOpt(':','P','password','<password/pathfile>','password or path for pubkey, sharedkey or special authentication options');
$params->addOpt(':','A','authfile','<filepath>','authentication file filepath form set default user, password parameters');
$params->addOpt(':','T','type','<test_type>','To specifie test type');
$params->addOpt(':','t','timeout','<second>','Timeout for script usage');
$params->addOpt(':','m','match','<regex>','To match specific value');
$params->addOpt(':','s','auto','<speed>','To specify a speed');
$params->addOpt(':','e','exclude','<regex>','To exclude 1 or more values');
$params->addOpt(':','o','ok','<treshold>','warning treshold (only for service)');
$params->addOpt(':','w','warning','<treshold>','warning treshold (not required for service)');
$params->addOpt(':','c','critical','<treshold>','critical treshold (not required for service)');
$params->addOpt(':','u','unknown','<treshold>','unknown treshold (only for service)');

$params->addOpt('','W','warning_ret',NULL,'return warning if error detected');
$params->addOpt('','C','critical_ret',NULL,'return critical if error detected');

$params->addOpt('','J','json',NULL,'return all data into json format');

$params->addUsage('-H <host> -U <user> -P <password -T <type> -w <treshold> -c <treshold>');
$params->addUsage('-H <host> [-U <user>] -A <authfile> -T <type> -w <treshold> -c <treshold>');

if( $params->getParam('help') )
{
    $params->print_help();
    exit($INT_STATUS['UNKNOWN']);
}

$mode_json = FALSE;
if( $params->getParam('json') )
{
    $mode_json = TRUE;
}

$o_type = $params->getParam('type');
if(!in_array($o_type,$type_list))
{
    $params_opt_str = $params->getParamOptionsString('type');
    $str = "Incorrect value $params_opt_str=$o_type.";
    $str.= " This value must be : ".implode(", ",$type_list);
    exit_state('UNKNOWN',$str);
}

if(!$mode_json)
{
    $required = array('warning','critical');
    if( $o_type != 'service' && $o_type != 'scheduled_task' && $o_type != 'smartctl')
    {
        foreach($required as $key)
        {
            $auth_params[$key] = $params->getParam($key,@$auth_file_data[$key]);
            $params_opt_str = $params->getParamOptionsString($key);
            if( empty($auth_params[$key]))
            {
                echo "Missing required option $params_opt_str\n\n";
                $params->print_usage();
                exit($INT_STATUS['UNKNOWN']);
            }
        }

        foreach(array('ok','unknown') as $key)
        {
            $auth_params[$key] = $params->getParam($key,@$auth_file_data[$key]);
            $params_opt_str = $params->getParamOptionsString($key);
            if( !empty($auth_params[$key]))
            {
                echo "Next option is not used for this plugin type $params_opt_str\n\n";
                $params->print_usage();
                exit($INT_STATUS['UNKNOWN']);
            }
        }
    }
}

$is_debug_mode = $params->getParam('debug');

$o_authfile_path = $params->getParam('authfile');
if(!empty($o_authfile_path))
{
    $params_opt_str = $params->getParamOptionsString('authfile');
    if( !file_exists($o_authfile_path) )
    {
        exit_state('UNKNOWN',"Unable to found $params_opt_str=$o_authfile_path");
    }
    $auth_file_data = file_get_contents($o_authfile_path);
    if( empty($auth_file_data))
    {
        exit_state('UNKNOWN',"Unable to open $params_opt_str=$o_authfile_path");
    }
    $auth_file_data = json_decode($auth_file_data,TRUE);
    if( !is_array($auth_file_data))
    {
        exit_state('UNKNOWN',"Unable to parse $params_opt_str=$o_authfile_path");
    }
}

if( $is_debug_mode)
{
    echo "File $o_authfile_path opened\n";
}

$auth_params = array();
foreach(array('host','user','password') as $key)
{
    $auth_params[$key] = $params->getParam($key,@$auth_file_data[$key]);
    $params_opt_str = $params->getParamOptionsString($key);
    if( empty($auth_params[$key]))
    {
        exit_state('UNKNOWN',"Missing required option $params_opt_str or authfile parameter $key");
    }
}

//Armement timeout
declare(ticks = 1);
function sig_handler($signo)
{
        global $timeout;
        exit_state("UNKNOWN","Timeout after $timeout seconds");
}

$timeout = $params->getParam('timeout',$DEFAULT_TIMEOUT);

pcntl_alarm($timeout);
pcntl_signal(SIGALRM, "sig_handler");

if( $is_debug_mode)
{
    echo "Timeout settings is alarm:$timeout, conn: $conn_timeout, curl: $curl\n";
}

$o_url = $params->getParam('uri',$o_server_url);
$http_params['key'] = $o_key;
$http_params['action'] = 'run';
$http_params['proto'] = $DEFAULT_TYPE;
$http_params['hostaddr'] = $auth_params['host'];
$http_params['user'] = $auth_params['user'];
//print_r($http_params);
$http_params['authentication'] = $auth_params['password'];

if(!$mode_json)
{
    $warning = $params->getParam('warning');
    $critical = $params->getParam('critical');
}
$type = $params->getParam('type');
try
{
    if( $type == 'load' )
    {
        if(!$mode_json)
        {
            validate_treshold_calcul_max($warning,$critical,100);
        }
        //Formula : (1- ((Nx - N0) / (Dx - D0))) x 100
        //see : https://learn.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2003/cc757283(v=ws.10)

        //Calcul du nom du fichier temporaire
        $uniq_ident =  md5(json_encode($http_params)."$type$mode_json");
        $filepath = sys_get_temp_dir() .'/'. basename(__FILE__) . $uniq_ident;

        //Connection au démon et récupération des données
        $select = 'PercentProcessorTime,Timestamp_Sys100NS';
        $query = 'select '.$select.' from Win32_PerfRawData_PerfOS_Processor where Name="_Total"';
        $command = "\$hash = @{};";
        $command.= "\$hash.hw=Get-CimInstance –ClassName Win32_Processor | Select-Object -Property NumberOfCores, NumberOfLogicalProcessors;";
        $command.= "\$hash.perfs=Get-CimInstance -Query '$query' | select -Property PercentProcessorTime,Timestamp_Sys100NS;";
        $command.= "\$hash | ConvertTo-Json";

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $from_host_data = $client->run($command);
        $client->close();

        $from_host_data = json_decode($from_host_data,TRUE);
        validateArray($from_host_data,'hw,perfs');
        if( isset($from_host_data['perfs']))
        {
            $hardware_info = $from_host_data['hw'];
            $from_host_data = $from_host_data['perfs'];
        }
        validateArray($from_host_data,$select);

        //chargement des données sauvées sur le disque
        $from_disk_data = array();
        if( is_file($filepath) )
        {
            $from_disk_data = @file_get_contents($filepath);
            $from_disk_data = json_decode($from_disk_data,TRUE);
            if( $from_disk_data == FALSE )
            {
                $from_disk_data = array();
            }
        }

        //sauvegarde des données sur le disques
        if( !@file_put_contents($filepath,json_encode($from_host_data)))
        {
            exit_state('UNKNOWN',"Unable to save data into tmp file $filepath");
        }

        //Calcul CPU usage percent
        if(empty($from_disk_data))
        {
            $cpu_usage= 'NAN';
            $result['state'] = 'OK';
            $result['reason'] = '. No enough data to calcul. Waiting next exec.';
        }
        else
        {
            $cpu_timepercent = $from_host_data['PercentProcessorTime'] - $from_disk_data['PercentProcessorTime'];
            $cpu_timestamp = $from_host_data['Timestamp_Sys100NS'] - $from_disk_data['Timestamp_Sys100NS'];
            $cpu_usage = (1 - ($cpu_timepercent / $cpu_timestamp) ) * 100;
            $cpu_usage = round($cpu_usage,2);
            $result = calcul_max_state($warning,$critical,$cpu_usage,'%');
        }
        $ncores = $hardware_info['NumberOfLogicalProcessors'];

        if($mode_json)
        {
            $output['ncores'] = $ncores;
            $output['cpu_usage'] = $cpu_usage;
            $output['last'] = $from_disk_data;
            $output['current'] = $from_host_data;
            echo json_encode($output,JSON_PRETTY_PRINT);
            exit(0);
        }

        $str = "$ncores CPU, average load $cpu_usage%";
        exit_state($result['state'],"$str".$result['reason']);
    }
    elseif( $type == 'mem' )
    {
        if(!$mode_json)
        {
            validate_treshold_calcul_max($warning,$critical,100);
        }

        //Connection au démon et récupération des données
        $select = 'FreePhysicalMemory,TotalVisibleMemorySize';
        $query = 'Select '.$select.' from Win32_OperatingSystem';
        $command = "Get-CimInstance -Query '$query' | select -Property $select | ConvertTo-Json";

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $from_host_data = $client->run($command);
        $client->close();

        $from_host_data = json_decode($from_host_data,TRUE);
        validateArray($from_host_data,$select);
        if($mode_json)
        {
            echo json_encode($from_host_data,JSON_PRETTY_PRINT);
            exit(0);
        }

        //Calcul Memory usage percents
        $mem_total = $from_host_data['TotalVisibleMemorySize'];
        $mem_usage = $mem_total - $from_host_data['FreePhysicalMemory'];
        $mem_perc = round($mem_usage/$mem_total*100,2);
        $mem_usage_MB = intval($mem_usage/1024);
        $mem_total_MB = intval($mem_total/1024);

        $result = calcul_max_state($warning,$critical,$mem_perc,'%');
        $str = "Physical Memory: $mem_perc% (${mem_usage_MB}MB/${mem_total_MB}MB)";
        exit_state($result['state'],"$str".$result['reason']);
    }
    elseif( $type == 'storage' )
    {
        if(!$mode_json)
        {
            $disk_select = $params->getParam('match');
            validate_treshold_calcul_max($warning,$critical,100);
        }

        $select = 'Capacity,DeviceID,DriveLetter,DriveType,FileSystem,FreeSpace,Label,Name';
        $query = 'Select '.$select.' from Win32_Volume where DriveType=3';
        $command = "Get-CimInstance -Query '$query' | select -Property $select | ConvertTo-Json";

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $from_host_data = $client->run($command);
        $client->close();

        $from_host_data = json_decode($from_host_data,TRUE);
        validateArray($from_host_data[0],$select);

        $global_state = 'OK';
        $str = '';

        $json_ret = array();
        foreach($from_host_data as $row)
        {
            if( preg_match('/Volume\{[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}\}/',$row['Name']))
            {
                continue;
            }
            if($mode_json)
            {
                $json_ret[] = $row;
            }

            $disk_key = $row['Name'];
            if( !empty($row['Label']))
            {
                $disk_key.= ' Label: '. $row['Label'];
            }
            if( !empty($disk_select) && stripos($disk_key,$disk_select) === FALSE && !@preg_match("#$disk_select#i",$disk_key))
            {
                continue;
            }

            $total = $row['Capacity'];
            $used = $total - $row['FreeSpace'];

            $ret_row = bytesToHuman($used,$total);
            $percent = $ret_row['percent']['usage'];
            $used = $ret_row['human']['usage'];
            $total = $ret_row['human']['total'];
            $unit = $ret_row['human']['unit'];

            $result = calcul_max_state($warning,$critical,$percent,'%');
            $global_state = calcul_state($global_state,$result['state']);
            $str .= $result['state']. ": $disk_key: $percent%used(${used}${unit}/${total}${unit})".$result['reason']."\n";

            $ret[] = $ret_row;
        }
        if($mode_json)
        {
            echo json_encode($json_ret,JSON_PRETTY_PRINT);
            exit(0);
        }
        if( count($ret) == 0)
        {
            exit_state('UNKNOWN',"Mountpoint $disk_select not FOUND");
        }
        elseif(count($ret) == 1 )
        {
            $disk = array_shift($ret);
            $str = trim($str);
            $str.= sprintf(" |used=%d;usedByte=%d;totalByte=%d\n",$disk['percent']['usage'],$disk['raw']['usage'],$disk['raw']['total']);
        }
        else
        {
            $str.= "NO GRAPH: Too many values for graphs, you have to choose one disk with -m; --match option\n";
        }
        echo $str;
        exit($global_state);
    }
    elseif( $type == 'uptime' )
    {
        if(!$mode_json)
        {
            validate_treshold_calcul_min($warning,$critical,0);
        }

        //Connection au démon et récupération des données
        //$select = 'OsUptime,CsCaption,CsDNSHostName,WindowsBuildLabEx,WindowsProductName,OsVersion,OsHardwareAbstractionLayer';
        $select = 'OsUptime,CsDNSHostName,WindowsProductName,WindowsVersion,OsVersion';
        $command = "Get-ComputerInfo -Property $select| select -Property $select | ConvertTo-Json";

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $from_host_data = $client->run($command);
        $client->close();

        $from_host_data = json_decode($from_host_data,TRUE);
        validateArray($from_host_data,$select);
        if($mode_json)
        {
            echo json_encode($from_host_data,JSON_PRETTY_PRINT);
            exit(0);
        }

        //Calcul uptime value
        $uptime = round($from_host_data['OsUptime']['TotalSeconds']);
        $hostname = $from_host_data['CsDNSHostName'];
        $kernel = sprintf("%s (%s) %s",$from_host_data['WindowsProductName'],$from_host_data['WindowsVersion'],$from_host_data['OsVersion']);

        $node = "$hostname $kernel";

        $period = secondsToPeriod($uptime);
        $str = "$node - up $period";
        $result = calcul_min_state($warning*60,$critical*60,$uptime,' minutes');
        exit_state($result['state'],"$str".$result['reason']);
    }
    elseif( $type == 'time' )
    {
        if( !$mode_json)
        {
            validate_treshold_calcul_max($warning,$critical,NULL);
        }
        $uts_offset_command = 'Get-Date -UFormat "%Z"';
        $ts_command = 'Get-Date -UFormat "%s"';

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $uts_offset = $client->run($uts_offset_command);
        $uts_calc = $uts_offset * 3600;

        //On récupére X fois l'information et on ne garde que celui qui a le taux d'erreur le plus faible.
        $date_calc = array();
        $date_min_error_idx = 0;
        $start_date = microtime(true);
        for($i=0;$i<3;$i++)
        {
            $read_date = trim($client->run($ts_command));
            $read_date -= $uts_calc;
            $end_date = microtime(true);
            $exec_time = $end_date-$start_date;
            $date_calc[$i]['local_first'] = $start_date;
            $date_calc[$i]['local_end'] = $end_date;
            $date_calc[$i]['local_estimated'] = ($exec_time/2) + $start_date;
            $date_calc[$i]['remote_date'] = $read_date;
            $date_calc[$i]['ratio_error'] = $end_date - $start_date;
            if( $date_calc[$i]['ratio_error'] < $date_calc[$date_min_error_idx]['ratio_error'] )
            {
                $date_min_error_idx = $i;
            }
            $date_calc[$i]['diff'] = $date_calc[$i]['local_estimated'] - $read_date;

            $start_date = $end_date;
        }
        $client->close();

        if($mode_json)
        {
            echo json_encode($date_calc,JSON_PRETTY_PRINT);
            exit(0);
        }

        $date_calc=$date_calc[$date_min_error_idx];

        if( $date_calc['local_first'] < $date_calc['remote_date'] && $date_calc['remote_date'] < $date_calc['local_end'])
        {
            $result['state'] = 'OK';
            $result['reason'] = ". Remote date between script starting end stopping date";
        }
        else
        {
            $result = calcul_max_state($warning,$critical,$date_calc['diff'],' seconds');
        }

            $str = sprintf("%s time differ by %.2f seconds (error treshold : %.2f)",$auth_params['host'],$date_calc['diff'],$date_calc['ratio_error']);
            exit_state($result['state'],"$str".$result['reason']);
    }
    elseif( $type == 'interface' )
    {
        $select_iface = '';
        if(!$mode_json)
        {
            validate_treshold_calcul_max($warning,$critical,100);

            $select_iface = $params->getParam('match');
            $param_speed = $params->getParam('speed');
        }
        $uniq_ident =  md5(json_encode($auth_params)."$type/$select_iface/$mode_json");
        $filepath = sys_get_temp_dir() .'/'. basename(__FILE__) . $uniq_ident;

        $loaded = array();
        if( is_file($filepath) )
        {
            $loaded = @file_get_contents($filepath);
            $loaded = json_decode($loaded,TRUE);
            if( $loaded == FALSE )
            {
                $loaded = array();
            }
        }

        //Connection au démon et récupération des données
        $iface_select = 'InstanceID,MacAddress,ifAlias,ifIndex,ifName,ifDesc';
        $iface_select.= ',Status,AdminStatus,MediaConnectionState,ifOperStatus,AdminLocked';
        $iface_select.= ',Speed,ReceiveLinkSpeed,TransmitLinkSpeed';

        $stats = 'Bytes,UnicastBytes,MulticastBytes,BroadcastBytes';
        $stats = explode(',',$stats);

        $stats_select = '';
        foreach($stats as $row)
        {
            $stats_select.= "Received$row,Sent$row,";
        }
        $stats_select.= "ReceivedPacketErrors,OutboundPacketErrors,OutboundDiscardedPackets";
        $stats_select = "InstanceID,$stats_select";

        $command = "\$hash = @{};";
        $command.= "\$hash.iface=Get-NetAdapter | select -Property $iface_select;";
        $command.= "\$hash.stats=Get-NetAdapterStatistics| select -Property $stats_select;";
        $command.= "\$hash | ConvertTo-Json";

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $timestamp = time();
        $from_host_data = $client->run($command);
        $from_host_data = json_decode($from_host_data,TRUE);
        $client->close();

        validateArray($from_host_data,array('iface','stats'));
        validateArray($from_host_data['iface'],$iface_select);
        validateArray($from_host_data['stats'],$stats_select);

        if( !is_array($from_host_data['iface']['InstanceID']))
        {
            $from_host_data['iface'] = array($from_host_data['iface']);
        }
        if( !is_array($from_host_data['stats']['InstanceID']))
        {
            $from_host_data['stats'] = array($from_host_data['stats']);
        }

        $merged_data = array();
        foreach($from_host_data['iface'] as $row)
        {
            $merged_data[$row['InstanceID']] = $row;
        }
        foreach($from_host_data['stats'] as $row)
        {
            $merged_data[$row['InstanceID']] = array_merge($merged_data[$row['InstanceID']],$row);
        }

        $parsed = array();
        $saving = array();
        foreach($merged_data as $row)
        {
            $ret_row = array();
            $iface_name = sprintf('%s. %s (%s)',$row['ifIndex'],$row['ifName'],$row['MacAddress']);
            if( !empty($select_iface))
            {
                //ifindex or digit 32772 in ifname ethernet_32772
                if( is_numeric($select_iface) && !preg_match("#_$select_iface$#",$row['ifName']))
                {
                    if( $select_iface != $row['ifIndex'] )
                    {
                        continue;
                    }
                }
                else
                {
                    $selected_mac = preg_replace('/[^a-z0-9]/','',strtolower($select_iface));
                    $readed_mac = preg_replace('/[^a-z0-9]/','',strtolower($row['MacAddress']));

                    //Macaddress or regex on ifname
                    if( $selected_mac != $readed_mac && !preg_match("#$select_iface#",$row['ifName']))
                    {
                        continue;
                    }
                }
            }
            $ret_row['time'] = $timestamp;
            $ret_row['rx_bytes'] = $row['ReceivedBytes'];
            $ret_row['tx_bytes'] = $row['SentBytes'];

            $ret_row['rx_unicast'] = $row['ReceivedUnicastBytes'];
            $ret_row['tx_unicast'] = $row['SentUnicastBytes'];

            $ret_row['rx_multicast'] = $row['ReceivedMulticastBytes'];
            $ret_row['tx_multicast'] = $row['SentMulticastBytes'];

            $ret_row['rx_broadcast'] = $row['ReceivedBroadcastBytes'];
            $ret_row['tx_broadcast'] = $row['SentBroadcastBytes'];

            $ret_row['rx_errs'] = $row['ReceivedPacketErrors'];
            $ret_row['tx_errs'] = $row['OutboundPacketErrors'];
            $ret_row['tx_discarded'] = $row['OutboundDiscardedPackets'];
            $saving[$iface_name]=$ret_row;

            $ret_row['speed'] = round($row['Speed']/1000/1000/1000);
            $ret_row['rx_speed'] = round($row['ReceiveLinkSpeed']/1000/1000/1000);
            $ret_row['tx_speed'] = round($row['TransmitLinkSpeed']/1000/1000/1000);
            $ret_row['Status'] = $row['Status'];
            $parsed[$iface_name] = $ret_row;
        }

        if( !@file_put_contents($filepath,json_encode($saving)))
        {
            exit_state('UNKNOWN',"Unable to save data into tmp file $filepath");
        }

        $calculated = array();
        foreach($parsed as $iface => $row)
        {
            foreach( explode(',','time,rx_bytes,rx_errs,tx_bytes,tx_errs,rx_unicast,tx_unicast,rx_multicast,tx_multicast,rx_broadcast,tx_broadcast') as $key)
            {
                if(isset($loaded[$iface][$key]))
                {
                    $calculated[$iface][$key] = $parsed[$iface][$key] - $loaded[$iface][$key];
                }
                else
                {
                    $calculated[$iface][$key] = 0;
                }
            }
            if( !empty($param_speed) && $param_speed != "auto" )
            {
                $calculated[$iface]['speed'] = $param_speed;
                $calculated[$iface]['rx_speed'] = $param_speed;
                $calculated[$iface]['tx_speed'] = $param_speed;
            }
            else
            {
                $calculated[$iface]['speed'] = $parsed[$iface]['speed'];
                $calculated[$iface]['rx_speed'] = $parsed[$iface]['rx_speed'];
                $calculated[$iface]['tx_speed'] = $parsed[$iface]['tx_speed'];

            }
        }

        if($mode_json)
        {
            $output['calculated'] = $calculated;
            $output['last'] = $loaded;
            $output['current'] = $saving;
            echo json_encode($output,JSON_PRETTY_PRINT);
            exit(0);
        }

        $str = '';
        $global_state = 'OK';
        $ifaces_stats = array();
        foreach($calculated as $iface => $row)
        {
            if(  $row['time']  == 0)
            {
                $row['time']  = 1;
            }
            $inBits = $row['rx_bytes']*8 / $row['time'];
            $outBits = $row['tx_bytes']*8 / $row['time'];

            $inUnicast = $row['rx_unicast']*8 / $row['time'];
            $outUnicast = $row['tx_unicast']*8 / $row['time'];

            $inMulticast = $row['rx_multicast']*8 / $row['time'];
            $outMulticast = $row['tx_multicast']*8 / $row['time'];

            $inBroadcast = $row['rx_broadcast']*8 / $row['time'];
            $outBroadcast = $row['tx_broadcast']*8 / $row['time'];

            $inErrors = $row['rx_errs'] / $row['time'];
            $outErrors = $row['tx_errs'] / $row['time'];

            $inStr = bitsToHumanStr($inBits);
            $outStr = bitsToHumanStr($outBits);

            if( $row['speed'] == -1 )
            {
                $state = 'UNKNOWN';
                $speed_str = ". No speed detected (Use -s, --speed option)";

                $warningBits = 0;
                $criticalBits = 0;
            }
            else
            {
                $warningBits_tx = ($row['tx_speed'] * 1000000 / 100) * $warning;
                $criticalBits_tx = ($row['tx_speed'] * 1000000 / 100) * $critical;

                $warningBits_rx = ($row['rx_speed'] * 1000000 / 100) * $warning;
                $criticalBits_rx = ($row['rx_speed'] * 1000000 / 100) * $critical;

                $result = calcul_max_state($warningBits_rx,$warningBits_rx,$inBits,'');
                $state = $result['state'];
                $result = calcul_max_state($warningBits_tx,$criticalBits_tx,$outBits,'');
                $state = calcul_state($state,$result['state']);
                $state = $STR_STATUS[$state];
                if($row['tx_speed'] == $row['rx_speed'])
                {
                    $speed_str = "speed=".$row['tx_speed']. "Mb/s";
                }
                else
                {
                    $speed_str = "speed=".$row['tx_speed'].'/'.$row['rx_speed']. "Mb/s";
                }
            }
            $str.= "$state: $iface: in=$inStr out=$outStr $speed_str\n";
            $global_state = calcul_state($state,$global_state);

            $retRow['in'] = $inBits;
            $retRow['out'] = $outBits;
            $retRow['warn'] = "$warningBits_tx/$warningBits_rx";
            $retRow['crit'] = "$criticalBits_tx/$warningBits_rx";
            $retRow['inErr'] = $inErrors;
            $retRow['outErr'] = $outErrors;
            $retRow['inUnicast'] = $inUnicast;
            $retRow['outUnicast'] = $outUnicast;
            $retRow['inMulticast'] = $inMulticast;
            $retRow['outMulticast'] = $outMulticast;
            $retRow['inBroadcast'] = $inBroadcast;
            $retRow['outBroadcast'] = $outBroadcast;
            $ifaces_stats[] = $retRow;
        }

        if( count($ifaces_stats) == 0)
        {
            exit_state('UNKNOWN',"Interface $select_iface not FOUND");
        }
        elseif(count($ifaces_stats) == 1 )
        {
            $iface = array_shift($ifaces_stats);
            $str = trim($str);
            $str.= sprintf("|data:in=%.2fdata:out=%.2f:warning:%s:critical:%s:inErrors:%.2f:outErrors:%.2f",$iface['in'],$iface['out'],$iface['warn'],$iface['crit'],$iface['inErr'],$iface['outErr']);
            $str.= sprintf("inUnicast:%d:outUnicast:%d:inMulticast:%d:outMulticast:%d:inBroadcast:%d:outBroadcast:%d",$iface['inUnicast'],$iface['outUnicast'],$iface['inMulticast'],$iface['outMulticast'],$iface['inBroadcast'],$iface['outBroadcast']);
        }
        else
        {
            $str.= "NO GRAPH: Too many values for graphs, you have to choose one disk with -m; --match option\n";
        }
        echo trim($str)."\n";
        exit($global_state);
    }
    elseif( $type == 'iodisk')
    {
    	//see : https://www.computerperformance.co.uk/powershell/perfmon/
    	//see : https://stackoverflow.com/questions/27272800/retrieve-average-disk-sec-per-transfer-using-wmi
    	//see : https://forums.powershell.org/t/win32-perfrawdata-perfdisk-logicaldisk/9876/4

        $select_disk='';
        $show_logical = TRUE;
        $show_physical = TRUE;
        $show_total = TRUE;
        if(!$mode_json)
        {
            validate_treshold_calcul_max($warning,$critical,100);
            $select_disk = $params->getParam('match');

            if( !empty($select_disk) || $select_disk === 0 )
            {
                if(stripos($select_disk,'total')!== FALSE)
                {
                    $show_total = TRUE;
                    $show_logical = FALSE;
                    $show_physical = FALSE;
                }
                else
                {
                    $show_logical = !is_numeric($select_disk);
                    $show_physical= is_numeric($select_disk);
                    $show_total = FALSE;
                }
            }
        }

        $uniq_ident =  md5(json_encode($auth_params)."$type/$select_disk/$mode_json");
        $filepath = sys_get_temp_dir() .'/'. basename(__FILE__) . $uniq_ident;

        $select = '*';
	    $queryP = 'Select '.$select.' from win32_PerfRawData_Perfdisk_PhysicalDisk';
        $queryL = 'Select '.$select.' from Win32_PerfRawData_PerfDisk_LogicalDisk';
        $command = "\$hash = @{};";
        if( $show_logical )
        {
            $command.= "\$hash.logical=Get-CimInstance -Query '$queryL' | select -Property $select;";
        }
        if( $show_physical || $show_total )
        {
            $command.= "\$hash.physical=Get-CimInstance -Query '$queryP' | select -Property $select;";
        }
        $command.= "\$hash | ConvertTo-Json";
        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $from_host_data = $client->run($command);
        $client->close();

        $from_host_data = json_decode($from_host_data,TRUE);
        if( $show_logical )
        {
            validateArray($from_host_data,'logical');
        }
        if( $show_physical )
        {
            validateArray($from_host_data,'physical');
        }

        $tmp = array();
        foreach($from_host_data as $type => $data)
        {
            foreach($data as $row)
            {
                if( $row['Name'] == '_Total' && $type == 'logical' )
                {
                    continue;
                }
                $row['type'] = $type;
                $tmp[] = $row;
            }
        }
        $from_host_data = $tmp;

        //chargement des données sauvées sur le disque
        $from_disk_data = array();
        if( is_file($filepath) )
        {
            $from_disk_data = @file_get_contents($filepath);
            $from_disk_data = json_decode($from_disk_data,TRUE);
            if( $from_disk_data == FALSE )
            {
                $from_disk_data = array();
            }
        }

        //sauvegarde des données sur le disques
        if( !@file_put_contents($filepath,json_encode($from_host_data)))
        {
            exit_state('UNKNOWN',"Unable to save data into tmp file $filepath");
        }

        $str = '';
        $global_state = 'OK';
        $ret = array();
        foreach($from_host_data as $key=>$data_row)
        {
            if( !$mode_json && (!empty($select_disk) || $select_disk !== NULL))
            {
                if( $show_total)
                {
                    if( $data_row['Name'] != '_Total' )
                    {
                        continue;
                    }
                }
                elseif( $show_logical && $data_row['type'] == 'logical')
                {
                    if( strtolower($data_row['Name']) != strtolower($select_disk) && !preg_match("/$select_disk/i",$data_row['Name']))
                    {
                        continue;
                    }
                }
                elseif( $show_physical && $data_row['type'] == 'physical' )
                {
                    $exp = explode(' ',$data_row['Name']);
                    if( $exp[0].'' != "$select_disk" )
                    {
                        continue;
                    }
                }
                else
                {
                    continue;
                }
            }

            $retRow = array();
            $retRow['name'] = $data_row['Name'];
            $percent_keys = 'PercentDiskReadTime,PercentDiskWriteTime,PercentDiskTime,PercentIdleTime';
            $queues_keys = 'AvgDiskQueueLength,AvgDiskReadQueueLength,AvgDiskWriteQueueLength';
            $bytesPersec_keys = 'DiskBytesPersec,DiskReadBytesPersec,DiskWriteBytesPersec';

            if( empty($from_disk_data) )
            {
                $all_keys = "$percent_keys,$queues_keys,$bytesPersec_keys";
                foreach(explode(',',$all_keys) as $p_key)
                {
                    $retRow[$p_key] = 0;
                }
            }
            else
            {
                $disk_row = $from_disk_data[$key];

                foreach(explode(',',$percent_keys) as $p_key)
                {
                    $n0 = $disk_row[$p_key];
                    $n1 = $data_row[$p_key];
                    $d0 = $disk_row['Timestamp_Sys100NS'];
                    $d1 = $data_row['Timestamp_Sys100NS'];

                    //N1 - N0 / D1 - D0
                    $retRow[$p_key] = (($n1-$n0)/($d1-$d0))*100;
                    $retRow[$p_key] = round($retRow[$p_key],2);
                }

                foreach(explode(',',$queues_keys) as $p_key)
                {
                    $tb = $data_row['Frequency_Sys100NS'];
                    $x0 = $disk_row[$p_key];
                    $x1 = $data_row[$p_key];
                    $y0 = $disk_row['Timestamp_Sys100NS'];
                    $y1 = $data_row['Timestamp_Sys100NS'];

                    //(TB(X1-X0)) / (Y1-Y0)
                    $retRow[$p_key] = ($tb*($x1-$x0)) / ($y1-$y0);
                    $retRow[$p_key] = round($retRow[$p_key],2);
                }
    /*
                $avg_keys='AvgDisksecPerRead,AvgDisksecPerTransfer,AvgDisksecPerWrite';
                foreach(explode(',',$avg_keys) as $p_key)
                {
                    $n0 = $disk_row[$p_key];
                    $n1 = $data_row[$p_key];
                    $d0 = $disk_row[$p_key.'_Base'];
                    $d1 = $data_row[$p_key.'_Base'];
                    $f = $data_row['Frequency_PerfTime'];

                    //((Nx - N0) / F) / (Dx - D0)
                    $retRow[$p_key] = (($n1-$n0)/$f)/($d1-$d0);
                    $retRow[$p_key] = round($result[$p_key],2);
                }
    */
                foreach(explode(',',$bytesPersec_keys) as $p_key)
                {
                    $n0 = $disk_row[$p_key];
                    $n1 = $data_row[$p_key];
                    $d0 = $disk_row['Timestamp_PerfTime'];
                    $d1 = $data_row['Timestamp_PerfTime'];
                    $f = $data_row['Frequency_PerfTime'];

                    //((Nx - N0) / ((Dx - D0)/ F)
                    $retRow[$p_key] = ($n1-$n0)/(($d1-$d0)/$f);
                    $retRow[$p_key] = round($retRow[$p_key],2);
                }
            }


            $disk_name = $retRow['name'];
            $used_percent = $retRow['PercentDiskTime'];
            $read_persec_human = bytesToHuman($retRow['DiskReadBytesPersec'],$retRow['DiskReadBytesPersec']+1);
            $write_persec_human = bytesToHuman($retRow['DiskWriteBytesPersec'],$retRow['DiskWriteBytesPersec']+1);

            $read_persec_human = $read_persec_human['human']['usage'] . ' '.$read_persec_human['human']['unit'].'/s';
            $write_persec_human = $write_persec_human['human']['usage'] . ' '.$write_persec_human['human']['unit'].'/s';

            $result = calcul_max_state($warning,$critical,$used_percent,'%');
            $global_state = calcul_state($global_state,$result['state']);

            $str.= $result['state']. ":$disk_name used $used_percent%. Read: $read_persec_human, Write: $write_persec_human\n";
            $ret[]=$retRow;
        }

        if($mode_json)
        {
            echo json_encode($ret,JSON_PRETTY_PRINT);
            exit(0);
        }

        if($mode_json)
        {
            echo json_encode($json_ret,JSON_PRETTY_PRINT);
            exit(0);
        }
        if( count($ret) == 0)
        {
            exit_state('UNKNOWN',"Mountpoint $select_disk not FOUND");
        }
        elseif(count($ret) == 1 )
        {
            $disk = array_shift($ret);
            $str = trim($str);

            $str.= sprintf(" |percent=%d;read=%d;write=%d;readQueue=%d;writeQueue=%d\n",
                $disk['PercentDiskTime'],
                $disk['DiskReadBytesPersec'],
                $disk['DiskWriteBytesPersec'],
                $disk['AvgDiskReadQueueLength'],
                $disk['AvgDiskWriteQueueLength']);
        }
        else
        {
            $str.= "NO GRAPH: Too many values for graphs, you have to choose one disk with -m; --match option\n";
        }
        echo $str;
        exit($global_state);
    }
    elseif( $type == 'smartctl')
    {
        $select = 'Active,InstanceName,PredictFailure,Reason';
        $command = "Get-CimInstance -Class 'MSStorageDriver_FailurePredictStatus' -Namespace 'ROOT\WMI' | select -Property $select | ConvertTo-Json -Depth 1";

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $from_host_data = $client->run($command);
        $client->close();

        $from_host_data = json_decode($from_host_data,TRUE);
        if( isset($from_host_data['Active']) )
        {
            $from_host_data = array($from_host_data);
        }
        validateArray($from_host_data[0],$select);

        if($mode_json)
        {
            echo json_encode($from_host_data,JSON_PRETTY_PRINT);
            exit(0);
        }

        $str = '';
        $global_state = 'OK';
        foreach($from_host_data as $row)
        {
            $msg = '';
            $status = 'OK';
            if( !$row['Active'])
            {
                $msg = 'Failure Prediction disabled';
                $status = 'WARNING';
            }
            elseif( $row['PredictFailure'] )
            {
                $msg = $row['Reason'];
                $msg.= ' disk Failures detected';
                $status = 'WARNING';
            }
            else
            {
                $msg = 'No error detected';
                $status = 'OK';
            }
            $global_state = calcul_state($global_state,$status);
            $str .= "$status : $msg on instance ".$row['InstanceName'];
        }
        echo "$str\n";
        exit($global_state);
    }
    elseif( $type == 'service')
    {
        //see next for info : https://learn.microsoft.com/en-us/dotnet/api/system.serviceprocess.servicecontroller.servicetype?view=dotnet-plat-ext-7.0

        //N/A by Microsoft, see next for other : https://learn.microsoft.com/en-us/dotnet/api/system.serviceprocess.servicecontrollerstatus?view=dotnet-plat-ext-7.0#fields
        $status_enum = array(1=>'Stopped','StartPending','StopPending','Running','ContinuePending','PausePending','Paused');

        //https://learn.microsoft.com/en-us/dotnet/api/system.serviceprocess.servicestartmode?view=dotnet-plat-ext-7.0#fields
        $start_enum = array('Boot','System','Automatic','Manual','Disabled');

        //Validate_treshold
        if(!$mode_json)
        {
            $ok = parse_string_treshold($params->getParam('ok'));
            $warn = parse_string_treshold($params->getParam('warning'));
            $crit = parse_string_treshold($params->getParam('critical'));
            $unk = parse_string_treshold($params->getParam('unknown'));
            validate_string_tresholds($ok,$warn,$crit,$unk,$status_enum);
        }

        $select = 'DisplayName,ServiceName,Status,StartType';
        $command = "Get-Service | select -Property $select| ConvertTo-Json";

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $from_host_data = $client->run($command);
        $client->close();

        $from_host_data = json_decode($from_host_data,TRUE);
        validateArray($from_host_data[0],$select);

        $select_services = $params->getParam('match');
        $exclude_services = $params->getParam('exclude');

        $services = array();
        $stats = array();
        $stats['total'] = 0;
        $stats['matched'] = 0;
        $stats['Running'] = 0;
        $stats['Paused'] = 0;
        $stats['Stopped'] = 0;
        $stats['Other'] = 0;
        foreach($from_host_data as $row)
        {
            $stats['total']++;
            if(!$mode_json)
            {
                if( !empty($exclude_services) && ($exclude_services == $row['ServiceName'] || preg_match("/$exclude_services/i",$row['ServiceName'])))
                {
                    continue;
                }
                elseif( !empty($select_services) && $select_services != $row['ServiceName'] && !preg_match("/$select_services/i",$row['ServiceName']))
                {
                    continue;
                }
            }
            $stats['matched']++;

            $retRow = $row;
            $retRow['Status'] = $status_enum[$row['Status']];
            $retRow['StartType'] = $start_enum[$row['StartType']];

            if( in_array($retRow['Status'],$stats) )
            {
                $stats[$retRow['Status']]++;
            }
            else
            {
                $stats['Other']++;
            }
            $services[] = $retRow;
        }

        if($mode_json)
        {
            echo json_encode($services,JSON_PRETTY_PRINT);
            exit(0);
        }

        //Calcul Tresholds
        $global_state = 'OK';
        $output = '';

        $stats['OK'] = 0;
        $stats['WARNING'] = 0;
        $stats['CRITICAL'] = 0;
        $stats['UNKNOWN'] = 0;
        foreach($services as $row)
        {
            if(calcul_string_treshold($ok,$row['Status']))
            {
                $curr_state = 'OK';
            }
            elseif(calcul_string_treshold($crit,$row['Status']))
            {
                $curr_state = 'CRITICAL';
            }
            elseif(calcul_string_treshold($warn,$row['Status']))
            {
                $curr_state = 'WARNING';
            }
            elseif(calcul_string_treshold($unk,$row['Status']))
            {
                $curr_state = 'UNKNOWN';
            }
            else
            {
                $curr_state = 'OK';
            }
            $stats[$curr_state]++;
            $output .= sprintf("%s : Found %s service '%s' (mode:%s) : %s\n",$curr_state,$row['Status'],$row['ServiceName'],$row['StartType'],$row['DisplayName']);
            $global_state=calcul_state($global_state,$curr_state);
        }
        $output = trim($output);
        echo "$output\n";
        exit($global_state);
    }
    elseif( $type == 'scheduled_task')
    {
        $error_state = 'CRITICAL';
        if( !$mode_json)
        {
            if( $params->getParam('warning_ret'))
            {
                $error_state = 'WARNING';
            }
            $select_task = $params->getParam('match');
            $exclude_task = $params->getParam('exclude');
        }

        //see next url for enumerations : https://learn.microsoft.com/en-us/windows/win32/taskschd/task-scheduler-enumerated-types
        $task_state_enum = array('unknown','disabled','queued','ready','running');

        //see next url fort LastTaskResult : https://learn.microsoft.com/en-us/windows/win32/taskschd/task-scheduler-error-and-success-constants?redirectedfrom=MSDN

        $selectTask = 'State,Author,Date,Source,TaskName,TaskPath';
        $selectInfo = 'LastRunTime,LastTaskResult,NextRunTime,NumberOfMissedRuns,TaskName,TaskPath';
        $command = "\$hash = @{};";
        $command.= "\$hash.task=Get-ScheduledTask | select -Property $selectTask;";
        $command.= "\$hash.info=Get-ScheduledTask | Get-ScheduledTaskInfo | select -Property $selectInfo;";
        $command.= "\$hash | ConvertTo-Json";

        $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
        $client->open();
        $from_host_data = $client->run($command);
        $client->close();
        $from_host_data = json_decode($from_host_data,TRUE);

        validateArray($from_host_data,'task,info');
        validateArray($from_host_data['task'][0],$selectTask);
        validateArray($from_host_data['info'][0],$selectInfo);

        $parsed = array();
        $task_failed = array();
        $task_success = array();
        foreach($from_host_data['task'] as $row)
        {
            $retRow = $row;
            $retRow['State'] = $task_state_enum[$retRow['State']];
            $parsed[$row['TaskPath']][$row['TaskName']] = $retRow;
        }
        foreach($from_host_data['info'] as $row)
        {
            if(!$mode_json)
            {
                if( !empty($exclude_task) && ($exclude_task == $row['TaskName'] || $exclude_task == $row['TaskPath'] || preg_match("/$exclude_task/i",$row['TaskPath'].$row['TaskName'])))
                {
                    continue;
                }
                elseif( !empty($select_task) && $select_task != $row['TaskName'] && $select_task != $row['TaskPath'] && !preg_match("/$select_task/i",$row['TaskPath'].$row['TaskName']))
                {
                    continue;
                }
            }

            $retRow = $row;

            if( $retRow['LastTaskResult']  < hexdec('0x80000000') )
            {
                $retRow['LastTaskResultSuccess'] = TRUE;
            }
            else
            {
                $retRow['LastTaskResultSuccess'] = FALSE;
            }

            foreach(array('LastRunTime','NextRunTime') as $key)
            {
                if( preg_match('/\d+/',$retRow[$key],$matches) )
                {
                    $retRow[$key] = $matches[0]/1000;
                }
                $retRow[$key.'Human'] = 'N/A';
                if( !empty($retRow[$key]))
                {
                    $retRow[$key.'Human'] = date('Y-m-d H:i:s',$retRow[$key]);
                }
            }

            $parsed[$row['TaskPath']][$row['TaskName']] = array_merge($parsed[$row['TaskPath']][$row['TaskName']],$retRow);
            if($retRow['LastTaskResultSuccess'])
            {
                $task_success[] = $parsed[$row['TaskPath']][$row['TaskName']];
            }
            else
            {
                $task_failed[] = $parsed[$row['TaskPath']][$row['TaskName']];
            }
        }

        if($mode_json)
        {
            echo json_encode(array_merge($task_failed,$task_success),JSON_PRETTY_PRINT);
            exit(0);
        }

        $nb_failed = count($task_failed);
        $nb_success = count($task_success);
        $nb_total = $nb_failed + $nb_success;

        if($nb_total == 0)
        {
            exit_state('UNKNOWN','Scheduled task not found');
        }
        elseif( $nb_failed == 0 )
        {
            $str = "All scheduled tasks are Success : $nb_success/$nb_success.\n";
            $state = 'OK';
        }
        else
        {
            $str = "Serveral scheduled tasks are FAILED. Success : $nb_success/$nb_total.\n";
            foreach($task_failed as $row)
            {
                $str.= sprintf("FAILED : Task %s%s has State:%s LastRunTime:%s NextRunTime:%s NumberOfMissedRuns:%d LastResult:%d\n",
                    $row['TaskPath'],
                    $row['TaskName'],
                    $row['State'],
                    $row['LastRunTimeHuman'],
                    $row['NextRunTimeHuman'],
                    $row['NumberOfMissedRuns'],
                    $row['LastTaskResult']);
            }
            $state = $error_state;
        }
        exit_state($state,$str);
    }
}
catch(Exception $ex)
{
    exit_state('UNKNOWN',$ex->getMessage());
}

function bitsToHumanStr($bits)
{
    $unitList = array('b/s','Kb/s','Mb/s','Gb/s');
    foreach($unitList as $unit)
    {
        if( $bits < 1000 )
        {
            return "$bits $unit";
        }
        else
        {
            $bits = round($bits/1000,3);
        }
    }
    return "$bits $unit";
}

function secondsToPeriod($seconds)
{
    $seconds = intval($seconds);
    $dtF = new DateTime("@0");
    $dtT = new DateTime("@$seconds");
    return $dtF->diff($dtT)->format('%a days, %h hours, %i minutes and %s seconds');
}

function validateArray($arr,$keys)
{
    if( !is_array($arr))
    {
        exit_state('UNKNOWN',"Unable to parse received data from host.");
    }
    if( !is_array($keys) )
    {
        $keys = explode(',',$keys);
    }
    foreach($keys as $key)
    {
        if( !array_key_exists($key,$arr))
        {
            exit_state('UNKNOWN',"Missing key '$key' from received data from host.");
        }
    }
}

function parse_string_treshold($treshold)
{
    if( empty($treshold))
    {
        return array();
    }
    return explode(',',$treshold);
}

function validate_string_tresholds($ok,$warning,$critical,$unknown,$enum_list)
{
    $sum_array = array();
    foreach(array_merge($ok,$warning,$critical,$unknown) as $value)
    {
        if(!isset($sum_array[$value]))
        {
            $sum_array[$value] = 1;
        }
        else
        {
            exit_state('UNKNOWN',"ok, warning, critical and unknown status must be differents");
        }
        if( !in_array($value,$enum_list) && preg_grep("/$value/i",$enum_list) === FALSE)
        {
            exit_state('UNKNOWN',sprintf('ok/warning/critical/unknown status %s must be one of this : %s',$value,implode(',',$enum_list)));
        }
    }
}

function calcul_string_treshold($tresholds,$data)
{
    if( empty($tresholds))
    {
        return FALSE;
    }
    foreach($tresholds as $search)
    {
        if( $search == $data || preg_match("/$search/i",$data))
        {
            return TRUE;
        }
    }
    return FALSE;
}

function validate_treshold_calcul_min($warning,$critical,$min_treshold=NULL)
{
    if( $warning < $critical )
    {
        exit_state('UNKNOWN',"Warning treshold $warning must be lower than critical treshold $critical");
    }
    if( $min_treshold != NULL && $warning < $min_treshold )
    {
        exit_state('UNKNOWN',"Tresholds must be lowest than $min_treshold");
    }
}

function validate_treshold_calcul_max($warning,$critical,$max_treshold=NULL)
{
    if( $warning > $critical )
    {
        exit_state('UNKNOWN',"Warning treshold $warning must be bigger than critical treshold $critical");
    }
    if( $max_treshold != NULL && $critical > $max_treshold )
    {
        exit_state('UNKNOWN',"Tresholds must be lowest than $max_treshold");
    }
}

function calcul_min_state($warning,$critical,$value,$unit='')
{
    global $INT_STATUS;

    $ret = array();
    if( $value <= $critical )
    {
        $ret['state'] = 'CRITICAL';
        $ret['reason'] = " < $critical".$unit;
    }
    else if( $value <= $warning )
    {
        $ret['state'] = 'WARNING';
        $ret['reason'] = " < $warning".$unit;
    }
    else
    {
        $ret['state'] = 'OK';
        $ret['reason'] = " > $warning".$unit;
    }
    return $ret;
}

function calcul_max_state($warning,$critical,$value,$unit='')
{
    global $INT_STATUS;

    $ret = array();
    if( $value >= $critical )
    {
        $ret['state'] = 'CRITICAL';
        $ret['reason'] = " > $critical".$unit;
    }
    else if( $value >= $warning )
    {
        $ret['state'] = 'WARNING';
        $ret['reason'] = " > $warning".$unit;
    }
    else
    {
        $ret['state'] = 'OK';
        $ret['reason'] = " < $warning".$unit;
    }
    return $ret;
}

function bytesToHuman($usage,$total,$bits=FALSE)
{
    $free = $total-$usage;
    $ret['raw']['usage'] = $usage;
    $ret['raw']['free']  =
    $ret['raw']['total'] = $total;

    $ret['percent']['usage'] = round(($usage/$total)*100);
    $ret['percent']['free']  = round(($free/$total)*100);

    if( $bits ){
      $multiplicator = 1000;
    }
    else
    {
      $multiplicator = 1024;
    }

          $ref = $total;
    if( $ref < pow(1000,1) * 10 )
    {
        $calcul = 1;
        $ret['human']['unit']   = 'B';
        $ret['human']['usage']  = round($usage/$calcul);
        $ret['human']['free']   = round($free/$calcul);
        $ret['human']['total']  = round($total/$calcul);
    }
    elseif( $ref < pow(1000,2) * 10 )
    {
        $calcul = pow($multiplicator,1);
        $ret['human']['unit']   = 'KB';
        $ret['human']['usage']  = round($usage/$calcul);
        $ret['human']['free']   = round($free/$calcul);
        $ret['human']['total']  = round($total/$calcul);
    }
    elseif( $ref < pow(1000,3) * 10 )
    {
        $calcul = pow($multiplicator,2);
        $ret['human']['unit']   = 'MB';
        $ret['human']['usage']  = round($usage/$calcul);
        $ret['human']['free']   = round($free/$calcul);
        $ret['human']['total']  = round($total/$calcul);
    }
    elseif( $ref < pow(1000,4) * 10 )
    {
        $calcul = pow($multiplicator,3);
        $ret['human']['unit']   = 'GB';
        $ret['human']['usage']  = round($usage/$calcul);
        $ret['human']['free']   = round($free/$calcul);
        $ret['human']['total']  = round($total/$calcul);
    }
    elseif( $ref < pow(1000,5) * 10 )
    {
        $calcul = pow($multiplicator,4);
        $ret['human']['unit']   = 'TB';
        $ret['human']['usage']  = round($usage/$calcul);
        $ret['human']['free']   = round($free/$calcul);
        $ret['human']['total']  = round($total/$calcul);
    }
    else
    {
        $calcul = pow($multiplicator,5);
        $ret['human']['unit']   = 'PB';
        $ret['human']['usage']  = round($usage/$calcul);
        $ret['human']['free']   = round($free/$calcul);
        $ret['human']['total']  = round($total/$calcul);
    }
    if( $bits )
    {
      $ret['human']['unit'] = ucfirst(strtolower($ret['human']['unit']));
    }
    return $ret;
}

function calcul_state($a,$b=null){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];
        $PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

        if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
        if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

        foreach($PRIORITIES_STATES as $value){
                $value = $INT_STATUS[$value];

                if( $value == $a || $value == $b ) return $value;
        }
        return $INT_STATUS["UNKNOWN"];
}

function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null )
        {
          $msg=trim($msg);
          echo "$state : $msg\n";
        }
        if( $usage ) print_usage(true);
        exit($ret);
}

class DaemonClientPwsh
{
    private $client;
    private $http_params;

    function __construct($o_url,$http_params,$timeout)
    {
        // Réglage des timeouts de http en fonction du timeout de l'alarm.
        $conn_timeout = 1;
        $curl_timeout = 1;
        if($timeout > 20)
        {
            $conn_timeout = intval($timeout*0.2);
            $curl_timeout = intval($timeout*0.75);
        }
        elseif( $timeout > 10 )
        {
            $conn_timeout = intval($timeout*0.3);
            $curl_timeout = $timeout - $conn_timeout - 1;
        }
        elseif( $timeout > 5)
        {
            $conn_timeout = intval($timeout*0.4);
            $curl_timeout = $timeout - $conn_timeout;
        }
        elseif( $timeout > 2)
        {
            $conn_timeout = intval($timeout*0.5);
            $curl_timeout = $timeout - $conn_timeout;
        }

        // Prépration de la connection http
        $client = new HttpClient($o_url);
        $client->setOptions(CURLOPT_CONNECTTIMEOUT,$conn_timeout);
        $client->setOptions(CURLOPT_TIMEOUT,$curl_timeout);
        $this->client = $client;
        $this->http_params = $http_params;
    }

    function open()
    {
        $this->client->open();
    }

    function run($command)
    {
        $http_params = $this->http_params;
        $http_params['command'] = $command;
        $client = $this->client;
        $raw_data = $client->post('',$http_params);
        $code = $client->getHttpCode();

        //Analyse des données reçues
        $data = json_decode($raw_data,TRUE);
        if( !is_array($data) )
        {
            throw new Exception("Code HTTP $code received. Empty or invalid data received from daemon.");
        }
        elseif(isset($data['code']) && $data['code'] != 200 )
        {
            throw new Exception(sprintf('Code %d - %s received from daemon. Reason : %s',$data['code'],$data['message'],$data['more_info']));
        }
        elseif( empty($data['stdout']) && isset($data['stderr']))
        {
            throw new Exception(sprintf('Error %s received from host %s',$data['stderr'],$http_params['hostaddr']));
        }
        elseif( empty($data['stdout']))
        {
            throw new Exception("Unable to parse received json data from daemon : ".$raw_data);
        }
        return $data['stdout'];
    }

    function close()
    {
        $this->client->close();
    }
}

class Parameters
{
    private $data = array();
    private $options = NULL;
    private $params = NULL;
    private $usage = array();

    function __construct()
    {
    }

    function addUsage($row)
    {
        $this->usage[] = $row;
    }

    static function getArrayBiggestWordSize($arr){
    	$biggest_size = 0;
            foreach($arr as $word){
                    $cur_size = strlen($word);
                    if( $biggest_size < $cur_size){
                            $biggest_size = $cur_size;
                    }
            }
    	return $biggest_size;
    }

    function print_usage($alone=TRUE)
    {
        $script = basename(__FILE__);
        $left = 'usage : ';
        $size_left = strlen($left);

        foreach($this->usage as $row)
        {
            echo str_pad($left,$size_left) . "$script $row [OPTIONS]\n";
            $left = '';
        }
        echo"\n";
        if($alone)
        {
            $row = "--help for more help";
            echo str_pad($left,$size_left) . "$script $row\n";
        }
    }

    function print_help()
    {
        $this->print_usage(FALSE);
        $opt = array();
        $desc = array();
        foreach($this->data as $key => $row)
        {
            $str = "\t-" . $this->getParamOptionsString($key);
            if( !empty($row['value']) )
            {
                $str.= '=<'.trim($row['value'],'<>').">";
            }
            $opt[] = $str;
            $desc[] = $row['descr'];
        }
        $size_left = self::getArrayBiggestWordSize($opt);
        foreach($opt as $idx => $left)
        {
            echo str_pad($left,$size_left) . "  ".$desc[$idx]."\n";
        }
    }

    public function getParamOptionsString($long_opt)
    {
        $row = $this->data[$long_opt];
        return "-".$row['short'] . ", --" . $row['long'];
    }

    public function addOpt($optional,$short_opt,$long_opt,$value_desc,$descr,$default = NULL)
    {
        $row = array();
        $row['optional'] = $optional;
        $row['short'] = $short_opt;
        $row['long'] = $long_opt;
        $row['value'] = $value_desc;
        $row['descr'] = $descr;
        $row['default'] = $default;
        $this->data[$long_opt] = $row;
    }

    private function getopt()
    {
        if( $this->options != NULL )
        {
            return;
        }

        $short_opts = '';
        $long_opts = array();

        foreach($this->data as $row)
        {
            $short_opts.= $row['short'].$row['optional'];
            $long_opts[] = $row['long'].$row['optional'];
        }
        $this->options = getopt($short_opts, $long_opts);
        $this->params = array();
        foreach($this->data as $row)
        {
            $req_opt = $row['optional'];
            $long_opt = $row['long'];
            $short_opt =  $row['short'];
            $default_opt = $row['default'];
            $key_name = $row['long'];

            if( $req_opt == '' )
            {
                if( isset($this->options[$long_opt]) || isset($this->options[$short_opt] ) )
                {
                    $value = TRUE;
                }
                else
                {
                    $value = FALSE;
                }
            }
            elseif( isset($this->options[$long_opt]))
            {
                $value = $this->options[$long_opt];
            }
            elseif( isset($this->options[$short_opt] ))
            {
                $value = $this->options[$short_opt];
            }
            else
            {
                continue;
                $value = $default_opt;
            }
            $this->params[$key_name] = $value;
        }
    }

    public function getParam($name,$default=NULL)
    {
        $this->getopt();
        if(isset($this->params[$name]) && is_numeric($this->params[$name]) && $this->params[$name] == 0)
        {
            return 0;
        }
        elseif( $default != NULL && empty($this->params[$name]) )
        {
            return $default;
        }
        elseif( !isset($this->params[$name]))
        {
            return NULL;
        }
        elseif(empty($this->params[$name]))
        {
            return FALSE;
        }
        return $this->params[$name];
    }
}
?>
